#!/usr/bin/env python
#  -*- mode: python; indent-tabs-mode: nil; -*- coding: iso-8859-1 -*-

"""

Candidates.py

Copyright 2009-2010 by Marcello Perathoner

Distributable under the GNU General Public License Version 3 or newer.

Utility class to find candidate files for conversion operations.

"""

import os.path
import fnmatch

from libgutenberg import GutenbergDatabase
from libgutenberg import GutenbergGlobals as gg


class Candidates (object):
    """ Class to get build candidates from the PG database.

    """

    def read_from_database (self, ebook):
        """ Read candidates from PG database. """

        candidates = []

        c = GutenbergDatabase.DB.get_cursor ()

        c.execute ("""
select filename, fk_encodings as encoding, fk_filetypes as type, filemtime as mtime, filesize as size, mediatype, generated
from files
  left join filetypes on (files.fk_filetypes = filetypes.pk)
where fk_books = %(ebook)s
  and fk_compressions = 'none'
  and diskstatus = 0
  and obsoleted = 0
order by fk_filetypes, fk_encodings, filemtime DESC""", {'ebook': ebook} )

        for row in c.fetchall ():
            file_ = GutenbergDatabase.xl (c, row)

            if (ebook > 10000 and file_.type == 'html'
                and not os.path.basename (file_.filename).startswith (str (ebook))):
                # must have the form 12345-h.htm (not eg. glossary.htm)
                continue

            adir = gg.archive_dir (ebook)
            if file_.filename.startswith (adir):
                file_.filename = file_.filename.replace (adir, 'files/%d' % ebook)
            elif file_.filename.startswith ('etext'):
                file_.filename = 'dirs/' + file_.filename

            file_.format = "%s/%s" % (file_.type, file_.encoding or 'unknown')

            file_.mediatype = str (gg.DCIMT (file_.mediatype, file_.encoding))

            candidates.append (file_)

        return candidates


    @staticmethod
    def filter_sort (typeglob_list, candidates, f):
        """ Filter and sort a list of candidates into preference order.

        typeglob_list is a list in the form:
          ('html/utf-8', 'html/iso-8859-*', 'html/*', '*/*')
        where the first entry is the preferred format.

        f (candidate) extracts type from candidate

        example: f = lambda x: x.format if not x.generated else 'skip_this'

        """

        result = []

        for typeglob in typeglob_list:
            for candidate in candidates:
                if fnmatch.fnmatch (f (candidate), typeglob):
                    if candidate not in result:
                        result.append (candidate)

        return result



#!/usr/bin/env python
#  -*- mode: python; indent-tabs-mode: nil; -*- coding: iso-8859-1 -*-

"""

QRCodeWriter.py

Copyright 2013 by Marcello Perathoner

Distributable under the GNU General Public License Version 3 or newer.

Writes QRCodes.

"""

import binascii
import os

from PIL import Image
import qrcode

from libgutenberg.Logger import info, exception
from ebookmaker import writers


DESKTOP_URL = 'https://www.gutenberg.org/ebooks/%d'
MOBILE_URL  = 'https://m.gutenberg.org/ebooks/%d'

BOXSIZE     = 4           # pixels per box
COLOR       = '000000ff'  # RGBA
BACKGROUND  = 'ffffff00'


class MyQRCode (qrcode.QRCode):
    """ QRCode with better image builder. """

    def __init__ (self):
        super (MyQRCode, self).__init__ (
            error_correction = qrcode.constants.ERROR_CORRECT_M,
            box_size         = 1,
            border           = 0,
        )


    def make_image (self, color, background):
        """
        Format QR Code as png image one pixel per qr box.

        color and background are RGBA values in hex, eg. '808080ff'.

        """

        clr = binascii.a2b_hex (color)
        bgc = binascii.a2b_hex (background)
        # bg_tuple = tuple (map (ord, bgc))

        # first flatten the data into a string
        # self.modules is an array of arrays of booleans
        rows = []
        for row in self.modules:
            rows.append (b''.join ([clr if x else bgc for x in row]))

        # the busy part of the qrcode, 1 pixel per box
        modules = self.modules_count
        return Image.frombytes ('RGBA', (modules, modules), b''.join (rows), 'raw', 'RGBA', 0, 1)


class Writer (writers.BaseWriter):
    """ QRCode image writer. """

    def build (self, job):
        """ Build qrcode sprite. """

        info ("Making   %s" % job.outputfile)

        try:
            qr = MyQRCode ()
            qr.add_data (DESKTOP_URL % job.ebook)
            qr.make (fit = True)
            qrcode1 = qr.make_image (COLOR, BACKGROUND)

            qr = MyQRCode ()
            qr.add_data (MOBILE_URL % job.ebook)
            qr.make (fit = True)
            qrcode2 = qr.make_image (COLOR, BACKGROUND)

            modules = qr.modules_count

            # build an empty image with room for 2 qrcodes, leave 1 column free between
            sizex, sizey = 2 * modules + 1, modules
            image = Image.new ('RGBA', (sizex, sizey), (0, 0, 0, 0))
            image.paste (qrcode1, (0, 0))
            image.paste (qrcode2, (modules + 1, 0))

            image = image.resize ((sizex * BOXSIZE, sizey * BOXSIZE))

            fn = os.path.join (job.outputdir, job.outputfile)
            with open (fn, 'wb') as fp:
                image.save (fp, 'png')

            info ("Done     %s" % job.outputfile)

        except Exception as what:
            exception ("Error building QR-Code: %s" % what)
            raise
